#include"cube.hpp"
#include"palette.hpp"
#include<algorithm>




Cube  cube_table[cube_table_size];


Point3D  Cube::current_point;


const int  Cube::width  = 48+font_size*12;
const int  Cube::height = 48;


void
Cube::
draw(Drawer&  dr, void*  data, int  item_index, int  x, int  y)
{
  auto&  h = cube_table[item_index];

  const int  n = (side_length*2);

  dr.draw_rect(0xFF00FFFF,x+4,y+4,n,n);

  transfer(h.thumbnail_,0,0,0,0,dr,x+4,y+4);

  dr.draw_unicode(h.name_.data(),black_color,x+n+8,y+(n/2));
}




Cube::
Cube():
name_(u"new_cube")
{
  std::memset(&elements_,0,sizeof(elements_));

  const int  n = (side_length*2);

  thumbnail_.resize(n,n);
}


Cube::
Cube(const Cube&  rhs)
{
  *this = rhs;
}



      Cube::Element&  Cube::get(int  x, int  y, int  z)      {return elements_[(size*size*z)+(size*y)+x];}
const Cube::Element&  Cube::get(int  x, int  y, int  z) const{return elements_[(size*size*z)+(size*y)+x];}


void
Cube::
operator=(const Cube&  rhs)
{
  name_ = rhs.name_;

  palette_ = rhs.palette_;

  std::memcpy(&elements_,&rhs.elements_,sizeof(elements_));

  thumbnail_ = rhs.thumbnail_;
}


void
Cube::
operator=(const Backup*  bk)
{
    if(bk)
    {
      std::memcpy(&elements_,bk,sizeof(*bk));
    }
}


Cube::Backup*
Cube::
create_backup() const
{
  auto  bk = new Backup;

  std::memcpy(bk,&elements_,sizeof(Backup));

  return bk;
}


void
Cube::
revolve_horizontally()
{
  Cube  tmp = *this;

    rep(z,size){
    rep(y,size){
    rep(x,size){
      get(size-1-y,x,z) = tmp.get(x,y,z);
    }}}
}


void
Cube::
revolve_vertically()
{
  Cube  tmp = *this;

    rep(z,size){
    rep(y,size){
    rep(x,size){
      get(x,size-1-z,y) = tmp.get(x,y,z);
    }}}
}




bool
locked;


void
Cube::
unlock()
{
  locked = false;
}


bool
Cube::
process(const Controller&  ctrl, Mark  mark, int  i)
{
    if(ctrl.test_pressing(Key::shift_flag))
    {
           if(ctrl.test_pressing(Key::down_flag)){if(current_point.z < (size-1)){current_point.z += 1;}}
      else if(ctrl.test_pressing(Key::up_flag  )){if(current_point.z           ){current_point.z -= 1;}}
    }

  else
    {
      const bool  flag = ctrl.test_pressing(Key::p_flag);

           if(ctrl.test_pressing(Key::up_flag   )){if(current_point.y           ){  current_point.y -= 1;  unlock();}}
      else if(ctrl.test_pressing(Key::left_flag )){if(current_point.x           ){  current_point.x -= 1;  unlock();}}
      else if(ctrl.test_pressing(Key::right_flag)){if(current_point.x < (size-1)){  current_point.x += 1;  unlock();}}
      else if(ctrl.test_pressing(Key::down_flag )){if(current_point.y < (size-1)){  current_point.y += 1;  unlock();}}
      else
        if(ctrl.test_pressing(Key::n_flag))
        {
          return false;
        }


        if(flag && !locked)
        {
          auto&  e = get(Cube::current_point.x,Cube::current_point.y,Cube::current_point.z);

          e.index = i;
          e.mark  = mark;


          locked = true;
        }
    }


  return true;
}




