#include"cuberenderer.hpp"
#include<algorithm>




namespace{


struct
Shell
{
  uint32_t  priority_number;

  int  focus;

  Point2D  point;

  const Cube::Element*  element;

  const Palette*  palette;

  Shell(const Palette&  pal, const Cube::Element&  e, int  x_, int  y_, int  z_):
  priority_number(x_+y_-z_),
  focus(0),
  point(x_,y_,z_),
  element(&e),
  palette(&pal)
  {
    point.x += Cube::get_rendparam().x_offset-Cube::side_length;
  }

};


std::vector<Shell>
buf;


bool
compare(const Shell&  a, const Shell&  b)
{
  return(a.priority_number < b.priority_number);
}


int
clamp(int  n)
{
  return((n <= 0)? 0:n);
}


template<typename  T>
void
draw(const Palette&  pal, const Cube::Element&  e, T&  dst, int  x_offset, int  y_offset)
{
    if(e.mark == Cube::Mark::paint)
    {
      const auto  base_pixel = pal.entries[e.index].pixel;

      int  r = to_r4(base_pixel);
      int  g = to_g4(base_pixel);
      int  b = to_b4(base_pixel);

      CubeRenderer::fill_top_plane(  dst,base_pixel                                    ,x_offset,y_offset);
      CubeRenderer::fill_front_plane(dst,  to_pixel16(clamp(r-2),clamp(g-2),clamp(b-2)),x_offset,y_offset);
      CubeRenderer::fill_right_plane(dst,  to_pixel16(clamp(r-4),clamp(g-4),clamp(b-4)),x_offset,y_offset);
    }

  else
    if(e.mark == Cube::Mark::paste)
    {
      auto&  pic = cube_table[e.index];

      transfer(pic.thumbnail_,0,0,0,0,dst,x_offset,y_offset);
    }
}

}


int&
CubeRenderer::
push(const Palette&  pal, const Cube::Element&  e, int  x, int  y, int  z)
{
  buf.emplace_back(pal,e,x*Cube::side_length,
                         y*Cube::side_length,
                         z*Cube::side_length);

  return buf.back().focus;
}


void
CubeRenderer::
clear()
{
  buf.clear();
}


void
CubeRenderer::
output(PixelMap16&  dst)
{
  std::stable_sort(buf.begin(),buf.end(),compare);

  PixelMap16  tmp(Cube::get_rendparam().width,Cube::get_rendparam().height);

    for(auto&  sh: buf)
    {
      draw(*sh.palette,*sh.element,tmp,sh.point.x,sh.point.y);
    }


  dst.fill();

  transform(tmp,dst);
}




void
CubeRenderer::
output(Drawer&  dr)
{
  std::stable_sort(buf.begin(),buf.end(),compare);

    for(auto&  sh: buf)
    {
      draw(*sh.palette,*sh.element,dr,sh.point.x,sh.point.y);

        if(sh.focus)
        {
            rep(i,Dir::END)
            {
              auto  dir = (Dir)i;

              draw_frame(dr,sh.point.x,sh.point.y,dir);
            }
        }
    }
}




