#ifndef IMP_HPP_INCLUDED
#define IMP_HPP_INCLUDED


#include<cstdint>
#include<cstdio>


enum class Dir
{
  top,
  bottom,
  left,
  right,
  front,
  back,

  END,

};


struct
Point3D
{
  int  x;
  int  y;
  int  z;

  constexpr Point3D(int  x_=0, int  y_=0, int  z_=0):
  x(x_), y(y_), z(z_)
  {
  }


  constexpr uint32_t  get_priority_number() const{return(x+y+z);}

  void  reset(int  x_=0, int  y_=0, int  z_=0)
  {
    x = x_;
    y = y_;
    z = z_;
  }

  Point3D  operator+(const Point3D&  rhs) const{return Point3D(x+rhs.x,y+rhs.y,z+rhs.z);}

  Point3D&  operator+=(const Point3D&  rhs)
  {
    x += rhs.x;
    y += rhs.y;
    z += rhs.z;

    return *this;
  }

  bool  operator==(const Point3D&  rhs) const
  {
    return((x == rhs.x) &&
           (y == rhs.y) &&
           (z == rhs.z));
  }

  void  print() const
  {
    printf("[print Point3D] X:%4d, Y:%4d, Z:%4d\n",x,y,z);
  }

};


struct
Point2D
{
  int  x;
  int  y;

  constexpr Point2D(int  x_=0, int  y_=0):
  x(x_), y(y_)
  {
  }

  constexpr Point2D(int  x_, int  y_, int  z_):
  x((x_  )-(y_  )   ),
  y((y_/2)+(x_/2)+z_)
  {
  }

  constexpr Point2D(const Point3D&  pt):
  x((pt.x  )-(pt.y  )     ),
  y((pt.y/2)+(pt.x/2)+pt.z)
  {
  }

  void  print() const
  {
    printf("[print Point2D] X:%4d, Y:%4d\n",x,y);
  }
};




struct
RenderingParameter
{
  int  width ;
  int  height;

  int  x_offset;

  constexpr RenderingParameter(int  x_width, int  y_width, int  z):
     width(Point2D(x_width,      0,0).x*2),
    height(Point2D(x_width,y_width,z).y  ),
  x_offset(Point2D(x_width,      0,0).x  )
  {
  }


  constexpr int  get_areasize() const
  {
    return(width*height);
  }

};




#endif




