#include"cube.hpp"
#include"widget.d/flexmenu.hpp"
#include"widget.d/lettermenu.hpp"
#include"widget.d/frame.hpp"
#include"widget.d/textboard.hpp"
#include"video.d/video.hpp"




Controller
ctrl;


namespace{


enum class Task
{
  choose_topmenuitem,
  choose_color,
  modify_colorelement,

  choose_cube,
  choose_cube_for_paste,
  make_cube_name,

  choose_mkhpmenuitem,
  paint_element,
  paste,

} task;


Cube::Mark
hmark;


int
proc_i;


int
display_timer;


std::u16string
display_message;


FlexMenu
top_menu({
u"キューブを作成",
u"リフレッシュ",
u"すべてのキューブを保存",
});


LetterMenu
letter_menu;


FlexMenu  cube_menu(8,10,Cube::width,Cube::height,Cube::draw,nullptr);

FlexMenu  cube_menu2(8,10,Cube::width,Cube::height,Cube::draw,nullptr);

FlexMenu  mkhp_menu({u"要素を塗る",
                     u"サムネイルを貼る",
                     u"消す",
                     u"コピー",
                     u"貼り付け",
                     u"水平回転",
                     u"垂直回転",
                     u"PNGで出力",
                    });


Frame    topmenu_frame;
Frame  paintmenu_frame;

Frame   cubemenu_frame;
Frame   cubemenu2_frame;


FlexMenu         color_menu(16,1,ColorEntry::width,ColorEntry::height,     ColorEntry::draw,nullptr);
FlexMenu  colorelement_menu(3,1,ColorElement::width,ColorElement::height,ColorElement::draw,nullptr);

Cube*
current_cube = cube_table;

Cube*
preview_cube;

ColorEntry*
current_color = cube_table->palette_.entries;




const char*  const
filename = "impdraw.dat";


void
save_cubes_all()
{
  FileStream  f(fopen(filename,"wb"));

  f.put(0);
  f.put(Cube::size);
  f.put_be16(cube_table_size);

    for(auto&  hp: cube_table)
    {
      hp.save(f);
    }


  display_timer = 32;

  display_message = u"SAVED";
}


void
load_cubes()
{
  auto  f = fopen(filename,"rb");

    if(f)
    {
      display_timer = 40;

      FileStream  fs(f);

      auto  ver = fs.get();

      auto  sz = fs.get();

        if(sz == Cube::size)
        {
          int  n = fs.get_be16();

          n = std::min(n,(int)cube_table_size);

          Cube*  dst = cube_table;

            while(n--)
            {
              dst++->load(fs);
            }


          display_message = u"読み込みました";
        }

      else
        {
          display_message = u"サイズが違います";
        }
    }
}


void
choose_topmenuitem()
{
    switch(top_menu.process(ctrl))
    {
      case(FlexMenu::Action::decide):
          switch(top_menu.get_item_index())
          {
            case(0):
              task = Task::choose_cube;
              break;
            case(1):
                for(auto&  hp: cube_table)
                {
                  hp.update();
                }


              display_timer = 20;
              display_message = u"REFRESHED";
              break;
            case(2):
              save_cubes_all();
              break;
          }
        break;
    }
}


void
modify_colorelement()
{
  auto&  ent = *static_cast<ColorEntry*>(colorelement_menu.get_data());

  int  rgb[] = {to_r4(ent.pixel),
                to_g4(ent.pixel),
                to_b4(ent.pixel)};

  int&  v = rgb[colorelement_menu.get_item_index()];

    switch(colorelement_menu.process(ctrl))
    {
      case(FlexMenu::Action::decide):
      case(FlexMenu::Action::cancel):
        task = Task::choose_color;

        proc_i = color_menu.get_item_index();
        colorelement_menu.set_cursor(false,false);
        break;
      case(FlexMenu::Action::left):
          if(v)
          {
            --v;
          }
        break;
      case(FlexMenu::Action::right):
          if(v < 15)
          {
            ++v;
          }
        break;
    }


  ent.pixel = to_pixel16(rgb[0],rgb[1],rgb[2]);
}


void
choose_color()
{
    switch(color_menu.process(ctrl))
    {
      case(FlexMenu::Action::decide):
        task = Task::modify_colorelement;

        current_color = &current_cube->palette_.entries[color_menu.get_item_index()];

        colorelement_menu.set_data(current_color);

        colorelement_menu.set_cursor(true,true);
        break;
      case(FlexMenu::Action::cancel):
        task = Task::choose_cube;

        current_cube->update();
        break;
      case(FlexMenu::Action::right):
        task = Task::choose_mkhpmenuitem;

        color_menu.set_cursor(false,true);
        mkhp_menu.set_cursor(true,true);
        break;
      case(FlexMenu::Action::up):
      case(FlexMenu::Action::down):
        current_color = &current_cube->palette_.entries[color_menu.get_item_index()];

        colorelement_menu.set_data(current_color);
        break;
      case(FlexMenu::Action::change):
        task = Task::choose_cube_for_paste;
        break;
    }
}


void
choose_cube()
{
    switch(cube_menu.process(ctrl))
    {
      case(FlexMenu::Action::decide):
        task = Task::choose_color;

        color_menu.set_cursor(true,true);
        mkhp_menu.set_cursor(false,false);
        colorelement_menu.set_cursor(false,false);

        current_cube = &cube_table[cube_menu.get_item_index()];
        current_color = current_cube->palette_.entries;

        colorelement_menu.set_data(current_color);

        color_menu.reset_item_index();
        color_menu.set_data(&current_cube->palette_);
        break;
      case(FlexMenu::Action::cancel):
        task = Task::choose_topmenuitem;
        break;
      case(FlexMenu::Action::start):
        task = Task::make_cube_name;

        current_cube = &cube_table[cube_menu.get_item_index()];

        letter_menu.reset_buffer(current_cube->name_);
        break;
    }
}


void
choose_cube_for_paste()
{
    switch(cube_menu2.process(ctrl))
    {
      case(FlexMenu::Action::decide):
        task = Task::paste;

        proc_i = cube_menu2.get_item_index();
        break;
      case(FlexMenu::Action::cancel):
        task = Task::choose_color;
        color_menu.set_cursor(true,true);
        mkhp_menu.set_cursor(false,false);
        break;
    }
}


void
make_cube_name()
{
    switch(letter_menu.process(ctrl))
    {
      case(LetterMenu::Action::finish):
        task = Task::choose_cube;

        current_cube->name_ = letter_menu.get_buffer();
        break;
    }
}


void
choose_mkhpmenuitem()
{
  static Cube  tmp;

  static int  ang;

  static Cube::Backup*  bk;

    switch(mkhp_menu.process(ctrl))
    {
      case(FlexMenu::Action::decide):
          switch(mkhp_menu.get_item_index())
          {
            case(0):
              task = Task::paint_element;
              hmark = Cube::Mark::paint;
              proc_i = color_menu.get_item_index();
              break;
            case(1):
              task = Task::choose_cube_for_paste;
              break;
            case(2):
              task = Task::paint_element;
              hmark = Cube::Mark::unuse;
              break;
            case(3):
              tmp = *current_cube;
              display_timer = 30;
              display_message = u"COPIED";
              break;
            case(4):
              *current_cube = tmp;
              break;
            case(5):
              current_cube->revolve_horizontally();
              break;
            case(6):
              current_cube->revolve_vertically();
              break;
            case(7):
              current_cube->save_as_png();
              display_timer = 30;
              display_message = u"SAVED AS PNG";
              break;
          }
        break;
      case(FlexMenu::Action::cancel):
        task = Task::choose_cube;

        break;
      case(FlexMenu::Action::left):
        task = Task::choose_color;
        color_menu.set_cursor(true,true);
        mkhp_menu.set_cursor(false,false);
        break;
    }
}


void
paint_element()
{
    if(ctrl.is_pressing_any())
    {
        if(!current_cube->process(ctrl,hmark,proc_i))
        {
          task = Task::choose_mkhpmenuitem;

          current_cube->update();
        }
    }

  else
    {
      Cube::unlock();
    }
}


void
paste()
{
    if(ctrl.is_pressing_any())
    {
        if(!current_cube->process(ctrl,Cube::Mark::paste,proc_i))
        {
          task = Task::choose_cube_for_paste;

          current_cube->update();
        }
    }

  else
    {
      Cube::unlock();
    }
}


void
preview()
{
    if(preview_cube)
    {
      current_cube->update();
    }
}


}


void
process(video::Device*  video_device)
{
  static uint32_t  last = 0;

  auto  now = SDL_GetTicks();

    if(now >= last+120)
    {
      last = now;

        switch(task)
        {
          case(Task::choose_topmenuitem ): choose_topmenuitem();   break;
          case(Task::choose_color       ): choose_color();         break;
          case(Task::modify_colorelement): modify_colorelement();  break;
          case(Task::choose_cube        ): choose_cube();          break;
          case(Task::choose_cube_for_paste): choose_cube_for_paste();          break;
          case(Task::choose_mkhpmenuitem): choose_mkhpmenuitem();            break;
          case(Task::make_cube_name     ): make_cube_name();       break;
          case(Task::paint_element      ): paint_element();       break;
          case(Task::paste              ): paste();       break;
        }
    }


  video_device->lock();

  auto  dr = video_device->get_drawer();

    switch(task)
    {
      case(Task::choose_topmenuitem):
        topmenu_frame.draw(dr);
        break;
      case(Task::choose_color):
        paintmenu_frame.draw(dr);
        current_cube->update(dr,true);
        break;
      case(Task::modify_colorelement):
        paintmenu_frame.draw(dr);
        current_cube->update(dr,true);
        break;
      case(Task::choose_cube):
        cubemenu_frame.draw(dr);
        break;
      case(Task::choose_cube_for_paste):
        cubemenu2_frame.draw(dr);
        current_cube->update(dr,true);
        break;
      case(Task::choose_mkhpmenuitem):
        paintmenu_frame.draw(dr);
        current_cube->update(dr,true);
        break;
      case(Task::paint_element):
        paintmenu_frame.draw(dr);
        current_cube->update(dr,true,true);
        break;
      case(Task::paste):
        cubemenu2_frame.draw(dr);
        current_cube->update(dr,true,true);
        break;
      case(Task::make_cube_name):
        letter_menu.draw(dr);
        break;
    }


    if(display_timer)
    {
      --display_timer;

      dr.fill_rect(                          0xFFFFFFFF,0,480-font_h-8,font_w*(display_message.size()+1)+8,font_h+8);
      dr.draw_unicode(display_message.data(),0x00000000,4,480-font_h-4                                             );
    }


  video_device->unlock();

  ctrl.clear();
}




void
initialize()
{
  task = Task::choose_topmenuitem;

  top_menu.set_cursor(true,true);
  cube_menu.set_cursor(true,true);
  cube_menu2.set_cursor(true,true);

    topmenu_frame.relative_point.set(0,0);
  paintmenu_frame.relative_point.set(Cube::get_rendparam().width,0);
   cubemenu_frame.relative_point.set(0,0);
   cubemenu2_frame.relative_point.set(Cube::get_rendparam().width,0);

    topmenu_frame.reset(u"MAIN MENU",{&top_menu},16);
  paintmenu_frame.reset(u"面を塗る",{&color_menu,&mkhp_menu},16);
   cubemenu_frame.reset(u"キューブを作成",{&cube_menu},16);
   cubemenu2_frame.reset(u"サムネイルを貼る",{&cube_menu2},16);


  paintmenu_frame.insert(&colorelement_menu,120,300);
  paintmenu_frame.cleanup();

  colorelement_menu.set_data(current_color);

  load_cubes();
}




